@@ -70,39 +70,6 @@ pub fn build(command: BuildCommand) -> Result<Vec<u8>, String> {
     let mut namespace: Namespace = Default::default();
     if let Some(ref mut deps) = manifest.dependencies {
         for (dependency_name, dependency_details) in deps.iter_mut() {
-            // Check if dependency is a git-based dependency.
-            let dep = match dependency_details {
-                Dependency::Simple(..) => {
-                    return Err(
-                        "Not yet implemented: Simple version-spec dependencies require a registry."
-                            .into(),
-                    );
-                }
-                Dependency::Detailed(dep_details) => dep_details,
-            };
-
-            // Download a non-local dependency if the `git` property is set in this dependency.
-            if let Some(git) = &dep.git {
-                let downloaded_dep_path = match dependency::download_github_dep(
-                    dependency_name,
-                    git,
-                    &dep.branch,
-                    &dep.version,
-                    offline_mode.into(),
-                ) {
-                    Ok(path) => path,
-                    Err(e) => {
-                        return Err(format!(
-                            "Couldn't download dependency ({:?}): {:?}",
-                            dependency_name, e
-                        ))
-                    }
-                };
-
-                // Mutate this dependency's path to hold the newly downloaded dependency's path.
-                dep.path = Some(downloaded_dep_path);
-            }
-
             compile_dependency_lib(
                 &this_dir,
                 dependency_name,
@@ -110,6 +77,7 @@ pub fn build(command: BuildCommand) -> Result<Vec<u8>, String> {
                 &mut namespace,
                 &mut dependency_graph,
                 silent_mode,
+                offline_mode,
             )?;
         }
     }
@@ -140,11 +108,41 @@ pub fn build(command: BuildCommand) -> Result<Vec<u8>, String> {
 fn compile_dependency_lib<'n, 'source, 'manifest>(
     project_file_path: &Path,
     dependency_name: &'manifest str,
-    dependency_lib: &Dependency,
+    dependency_lib: &mut Dependency,
     namespace: &mut Namespace<'source>,
     dependency_graph: &mut HashMap<String, HashSet<String>>,
     silent_mode: bool,
+    offline_mode: bool,
 ) -> Result<(), String> {
+    let mut details = match dependency_lib {
+        Dependency::Simple(..) => {
+            return Err(
+                "Not yet implemented: Simple version-spec dependencies require a registry.".into(),
+            )
+        }
+        Dependency::Detailed(ref mut details) => details,
+    };
+    // Download a non-local dependency if the `git` property is set in this dependency.
+    if let Some(ref git) = details.git {
+        let downloaded_dep_path = match dependency::download_github_dep(
+            dependency_name,
+            git,
+            &details.branch,
+            &details.version,
+            offline_mode.into(),
+        ) {
+            Ok(path) => path,
+            Err(e) => {
+                return Err(format!(
+                    "Couldn't download dependency ({:?}): {:?}",
+                    dependency_name, e
+                ))
+            }
+        };
+
+        // Mutate this dependency's path to hold the newly downloaded dependency's path.
+        details.path = Some(downloaded_dep_path);
+    }
     let dep_path = match dependency_lib {
         Dependency::Simple(..) => {
             return Err(
@@ -180,7 +178,7 @@ fn compile_dependency_lib<'n, 'source, 'manifest>(
         }
     };
 
-    let manifest_of_dep = read_manifest(&manifest_dir)?;
+    let mut manifest_of_dep = read_manifest(&manifest_dir)?;
 
     let main_path = {
         let mut code_dir = manifest_dir.clone();
@@ -201,20 +199,18 @@ fn compile_dependency_lib<'n, 'source, 'manifest>(
     );
     let mut dep_namespace: Namespace = Default::default();
 
-    // The part below here is just a massive shortcut to get the standard library working
-    if let Some(ref deps) = manifest_of_dep.dependencies {
-        for (dependency_name, dependency_lib) in deps {
+    if let Some(ref mut deps) = manifest_of_dep.dependencies {
+        for (dependency_name, ref mut dependency_lib) in deps {
             // to do this properly, iterate over list of dependencies make sure there are no
             // circular dependencies
-            //return Err("Unimplemented: dependencies that have dependencies".into());
             compile_dependency_lib(
                 &manifest_dir,
                 dependency_name,
                 dependency_lib,
-                // give it a cloned namespace, which we then merge with this namespace
                 &mut dep_namespace,
                 dependency_graph,
                 silent_mode,
+                offline_mode,
             )?;
         }
     }
